//
//  XXXBaseDAO.m
//  SQLiteDemo
//
//  Created by  SQLite+ on 10-4-13.
//  Copyright http://www.iosxtools.com All rights reserved.
//
#import "XXXBaseDAO.h"
@interface XXXBaseDAO ()
@property(nonatomic,readwrite)NSInteger count;/*numbers of record in table*/
/*@property(nonatomic,strong)FMDatabase *db;*/
@property(nonatomic,strong)FMDatabaseQueue *queue;
/*fetch records Array based on FMResultSet */
- (NSArray*)sqlQueryFetch:(FMResultSet*)rs;
- (id)findByModel:(id)model;
@end
@implementation XXXBaseDAO
/*init database */
- (id)init{
    self = [super init];
    if(self){
        _queue = [XXXDBManager sharedXXXDBManager].queue;
        if(_queue==0x00){
            return  nil;
        }
    }
	return self;
}

/*return record total number*/
- (NSInteger)rowNumbers{
    if(!self.tableName){
        return 0;
    }
    NSString *sql =[NSString stringWithFormat:@"SELECT count(*) as count FROM %@", self.tableName];
    _count = 0;
    [self.queue inDatabase:^(FMDatabase *db){
        FMResultSet *rs = [db executeQuery:sql];
        if([rs next]) {
            _count  = [rs intForColumnIndex:0];
        }
        [rs close];
    }];
    return _count;
}
- (NSInteger)sumOf:(NSString*)fieldName
{
    if(!self.tableName){
        return 0;
    }
    NSString *sql =[NSString stringWithFormat:@"SELECT sum(%@) as count FROM %@",fieldName,self.tableName];
    _count = 0;
    [self.queue inDatabase:^(FMDatabase *db){
        FMResultSet *rs = [db executeQuery:sql];
        if([rs next]) {
            _count  = [rs intForColumnIndex:0];
        }
        [rs close];
    }];
    return _count;
}
- (NSInteger)pageNumberWithSize:(NSInteger)pageSize;
{
    if(!self.tableName){
        return 0;
    }
    XXXAssert(pageSize>0);
    NSString *sql =[NSString stringWithFormat:@"SELECT count(*) as count FROM %@", self.tableName];
    
    __block int page = 0;
    [self.queue inDatabase:^(FMDatabase *db){
        FMResultSet *rs = [db executeQuery:sql];
        _count = 0;
        if([rs next]) {
            _count  = [rs intForColumnIndex:0];
        }
        [rs close];
        if(_count>0){
            page =  ceil(_count/pageSize);
        }
    }
    ];
    return page;
}
/*insert a record*/
- (BOOL)insert:(id)model{
    if(!model){
        return NO;
    }
	if([self findByKey:model]){
		DLog(@"info:add %@ exsist!\n",self.tableName);
		return [self update:model];
	}
    NSMutableString *vals = [[NSMutableString alloc]initWithCapacity:10];
    NSInteger fieldCount = [self.fldList count];
    for(NSInteger i =0 ;i < fieldCount;i++){
        if(i!=fieldCount-1){
            [vals appendString:@"?,"];
        }
        else{
            [vals appendString:@"?"];
        }
    }
    NSString *fieldString = [self.fldList componentsJoinedByString:@","];
    NSString *sql = [NSString stringWithFormat:@"INSERT INTO %@ (%@) VALUES ( %@ )",self.tableName,fieldString,vals];
    DLog(@"insert sql=%@",sql)
    NSMutableArray *args=[[NSMutableArray alloc]initWithCapacity:10];
    for(NSInteger i =0 ;i < fieldCount;i++){
        NSString *mapkey = [self.fldList objectAtIndex:i];
        id value = [model valueForKey:mapkey];
        if(!value){
            value = [NSNull null];
        }
        [args addObject:value];
    }
    __block BOOL isOK = NO;
    [self.queue inDatabase:^(FMDatabase *db){
        isOK = [db executeUpdate:sql withArgumentsInArray:args];
        DLog("insert lastErrorMessage =%@",[db lastErrorMessage]);
    }];
    return isOK;
}
/*update a record*/
- (BOOL)update:(id)model{
    if(!model){
        return NO;
    }
    if(![self findByKey:model]){
		DLog(@"info:no exsist update Model!\n");
        return NO;
	}
    NSMutableString *wheres = [[NSMutableString alloc]initWithCapacity:10];
    NSInteger keyCount = [self.keyList count];
    NSMutableString *fieldVals = [[NSMutableString alloc]initWithCapacity:10];
    NSMutableArray *args=[NSMutableArray arrayWithCapacity:5];
    NSInteger fieldCount = [self.fldExcludeKeyList count];
    for(NSInteger i =0 ;i < fieldCount;i++){
        NSString *fName = [self.fldExcludeKeyList objectAtIndex:i];
        if(i!=fieldCount-1){
            [fieldVals appendFormat:@"%@ = ? , ",fName];
        }
        else{
            [fieldVals appendFormat:@"%@ = ? ",fName];
            
        }
        id value = [model valueForKey:fName];
        if(!value){
            value = [NSNull null];
        }
        [args addObject:value];
    }
    if(fieldCount<=0){
        for(NSInteger i =0 ;i < keyCount;i++){
            NSString *fName = [self.keyList objectAtIndex:i];
            if(i!=keyCount-1){
                
                [fieldVals appendFormat:@"%@ = ? , ",fName];
            }
            else{
                [fieldVals appendFormat:@"%@ = ? ",fName];
                
            }
            id value = [model valueForKey:fName];
            if(!value){
                value = [NSNull null];
            }
            [args addObject:value];
        }
    }
    
    for(NSInteger i =0 ;i < keyCount;i++){
        NSString *fName = [self.keyList objectAtIndex:i];
        if(i!=keyCount-1){
            
            [wheres appendFormat:@"%@ = ? AND ",fName];
        }
        else{
            [wheres appendFormat:@"%@ = ? ",fName];
            
        }
        id value = [model valueForKey:fName];
        if(!value){
            value = [NSNull null];
        }
        [args addObject:value];
    }
    NSString *sql = [NSString stringWithFormat:@"UPDATE %@ SET %@ WHERE %@ ",self.tableName,fieldVals,wheres];

    DLog(@"update sql=%@",sql);
    
    __block BOOL isOK = NO;
    [self.queue inDatabase:^(FMDatabase *db){
        isOK = [db executeUpdate:sql withArgumentsInArray:args];
        DLog("update lastErrorMessage =%@",[db lastErrorMessage]);
    }];
    return isOK;

}
/*delete a record*/
- (BOOL)delete:(id)model{
    if(!model){
        return NO;
    }
	if(![self findByKey:model]){
		DLog(@"info:no exsist delete model!\n");
		return NO;
	}
    NSMutableString *wheres = [[NSMutableString alloc]initWithCapacity:10];
    NSInteger keyCount = [self.keyList count];
    for(NSInteger i =0 ;i < keyCount;i++){
        NSString *fName = [self.keyList objectAtIndex:i];
        if(i!=keyCount-1){
            
            [wheres appendFormat:@"%@ = ? AND ",fName];
        }
        else{
            [wheres appendFormat:@"%@ = ? ",fName];
            
        }
    }
    NSString *sql = [NSString stringWithFormat:@"DELETE FROM  %@ WHERE %@ ",self.tableName,wheres];
    NSMutableArray *args=[[NSMutableArray alloc]initWithCapacity:10];
    for(NSInteger i =0 ;i < keyCount;i++){
        NSString *mapkey = [self.keyList objectAtIndex:i];
        id value = [model valueForKey:mapkey];
        [args addObject:value];
    }
    DLog(@"delete sql=%@",sql); 
    __block BOOL isOK = NO;
    [self.queue inDatabase:^(FMDatabase *db){
        isOK = [db executeUpdate:sql withArgumentsInArray:args];
        DLog("delete lastErrorMessage =%@",[db lastErrorMessage]);
    }];
    return isOK;
}

/*execute  sql update in table*/
- (BOOL)sqlUpdate:(NSString*)sql{
    __block BOOL isOK = NO;
    [self.queue inDatabase:^(FMDatabase *db){
        isOK = [db executeUpdate:sql];
    }];
    return isOK;
}
/*execute  sql update in table*/
- (BOOL)sqlUpdate:(NSString*)sql withArgumentsInArray:(NSArray*)args{
    
    __block BOOL isOK = NO;
    [self.queue inDatabase:^(FMDatabase *db){
        if(args.count>0){
            isOK =  [db executeUpdate:sql withArgumentsInArray:args];
        }
        else{
            isOK =   [db executeUpdate:sql withArgumentsInArray:nil];
        }
    }];
    return isOK;

    
}
/*execute  sql update in table*/
- (BOOL)sqlUpdate:(NSString*)sql withParameterDictionary:(NSDictionary*)dics{
    NSInteger count =dics.allKeys.count;
    
    __block BOOL isOK = NO;
    [self.queue inDatabase:^(FMDatabase *db){
        if(!dics || count==0){
            isOK =  [db executeUpdate:sql];
        }
        else{
            isOK = [db executeUpdate:sql withParameterDictionary:dics];
        }
    }
    ];
    return isOK;
   
}
/*execute  sql query in table*/
- (NSArray*)sqlQuery:(NSString*)sql{
    __block NSArray *modles;
    [self.queue inDatabase:^(FMDatabase *db){
        FMResultSet *rs = [db executeQuery:sql];
        modles = [self sqlQueryFetch:rs];
        [rs close];
    }
    ];
    return  modles;
}
/*execute  sql query in table*/
- (NSArray*)sqlQuery:(NSString*)sql  withArgumentsInArray:(NSArray*)args{
    
    __block NSArray *modles;
    [self.queue inDatabase:^(FMDatabase *db){
        FMResultSet *rs;
        if(args.count>0){
            rs = [db executeQuery:sql withArgumentsInArray:args];
        }
        else{
            rs = [db executeQuery:sql];
        }
        modles = [self sqlQueryFetch:rs];
        [rs close];
    }
    ];
    return  modles;
}
/*execute  sql query in table*/
- (NSArray*)sqlQuery:(NSString*)sql  withParameterDictionary:(NSDictionary*)dics{
    NSInteger count =dics.allKeys.count;
    if(!dics || count==0){
        return  [self sqlQuery:sql];
    }
    __block NSArray *modles;
    [self.queue inDatabase:^(FMDatabase *db){
        FMResultSet *rs = [db executeQuery:sql withParameterDictionary:dics];
        modles = [self sqlQueryFetch:rs];
        [rs close];
    }
    ];
    return  modles;
}
/*delete all record*/
- (BOOL)clearAll{
    NSString *sql = [NSString stringWithFormat:@"DELETE FROM  %@ ",self.tableName];
    DLog(@"clearAll sql=%@",sql);
    __block BOOL isOK = NO;
    [self.queue inDatabase:^(FMDatabase *db){
        isOK = [db executeUpdate:sql];
        DLog("clearAll lastErrorMessage =%@",[db lastErrorMessage]);
    }];
    return isOK;
}
/*fetch all record*/
- (NSArray*)findAll{
    NSString *sql = [NSString stringWithFormat:@"SELECT * FROM  %@  ",self.tableName];
    DLog(@"findAll sql=%@",sql);
    __block NSArray *modles;
    [self.queue inDatabase:^(FMDatabase *db){
        FMResultSet *rs = [db executeQuery:sql];
        modles = [self sqlQueryFetch:rs];
        [rs close];
    }];
    return  modles;
}
/*fetch record By Primary Key Field*/
- (id)findByKey:(id)key{
    if([key isKindOfClass:[NSClassFromString(self.tableName) class]]){
        return  [self findByModel:key];
    }
    else if([key isKindOfClass:[NSDictionary class]]) {
        id retModel = [[NSClassFromString(self.tableName) alloc] init];
        NSInteger keyCount = self.keyList.count;
        for(NSInteger i =0 ;i < keyCount;i++){
            NSString *mapkey = [self.keyList objectAtIndex:i];
            id value = [key valueForKey:mapkey];
            [retModel setValue:value forKey:mapkey];
        }
        return  [self findByModel:retModel];
        
    }
    else{
        id retModel = [[NSClassFromString(self.tableName) alloc] init];
        NSInteger keyCount = self.keyList.count;
        for(NSInteger i =0 ;i < keyCount;i++){
            NSString *mapkey = [self.keyList objectAtIndex:i];
            [retModel setValue:key forKey:mapkey];
        }
        return  [self findByModel:retModel];
    }
    
}
- (id)findByModel:(id)model{
    if(!model){
        return nil;
    }
    NSInteger keyCount = [self.keyList count];
    if(keyCount<=0){
        DLog(@"table no primary key field!");
        return nil;
    }
    NSMutableString *wheres = [[NSMutableString alloc]initWithCapacity:10];
    for(NSInteger i =0 ;i < keyCount;i++){
        NSString *fName = [self.keyList objectAtIndex:i];
        if(i!=keyCount-1){
            
            [wheres appendFormat:@"%@ = ? AND ",fName];
        }
        else{
            [wheres appendFormat:@"%@ = ? ",fName];
            
        }
    }
    
    NSString *sql = [NSString stringWithFormat:@"SELECT * FROM  %@  WHERE %@ ",self.tableName,wheres];
    NSMutableArray *args=[[NSMutableArray alloc]initWithCapacity:10];
    for(NSInteger i =0 ;i < keyCount;i++){
        NSString *mapkey = [self.keyList objectAtIndex:i];
        id value = [model valueForKey:mapkey];
        [args addObject:value];
    }
    DLog(@"findByKey sql=%@",sql);
    
    __block id retModel = nil;
    [self.queue inDatabase:^(FMDatabase *db){
        FMResultSet *rs = [db executeQuery:sql withArgumentsInArray:args];
        if ([rs next]) {
            retModel = [[NSClassFromString(self.tableName) alloc] init];
            for(NSString  *mapkey in self.fldList){
                id obj=[rs objectForColumnName: mapkey];
                if([obj isEqual:[NSNull null]]){
                    continue;
                }
                [retModel setValue:obj forKey:mapkey];
            }
            [rs close];
        }
    }];
    return retModel;
}
/*fetch records for given attributes*/
- (NSArray*)findByAttributes:(NSDictionary*)Attributes{
    NSMutableString *wheres = [[NSMutableString alloc]initWithCapacity:10];
    NSInteger keyCount = [Attributes.allKeys count];
    if(keyCount<=0){
        DLog(@"findByAttributes :  attribute parameter is null  !");
        return nil;
    }
    for(NSInteger i =0 ;i < keyCount;i++){
        NSString *fName = [Attributes.allKeys objectAtIndex:i];
        if(i!=keyCount-1){
            [wheres appendFormat:@"%@ = :%@ AND ",fName,fName];
        }
        else{
            [wheres appendFormat:@"%@ = :%@ ",fName,fName];
        }
    }
    NSString *sql = [NSString stringWithFormat:@"SELECT * FROM  %@  WHERE %@ ",self.tableName,wheres];
    DLog(@"findByAttributes sql=%@",sql);
    
    __block NSArray *modles;
    [self.queue inDatabase:^(FMDatabase *db){
        FMResultSet *rs = [db executeQuery:sql withParameterDictionary:Attributes];
        modles = [self sqlQueryFetch:rs];
        [rs close];
    }];
    return  modles;
}
/*fetch records based on the index of pages*/
- (NSArray*)findByPage:(NSInteger)index pageSize:(NSInteger)pageSize{
    NSString *sql = [NSString stringWithFormat:@"SELECT * FROM  %@  LIMIT %ld OFFSET %ld ",self.tableName,pageSize,pageSize*index];
    DLog(@"findByPage sql=%@ pageindex=%ld",sql,index);
    
    
    __block NSArray *modles;
    [self.queue inDatabase:^(FMDatabase *db){
        FMResultSet *rs = [db executeQuery:sql];
        modles = [self sqlQueryFetch:rs];
        [rs close];
    }];
    return  modles;

}
- (NSArray*)sqlQueryFetch:(FMResultSet*)rs{
    NSMutableArray *modles  = [[NSMutableArray alloc]initWithCapacity:10];
    while ([rs next]) {
        id model = [[NSClassFromString(self.tableName) alloc] init];
        for(NSString  *mapkey in self.fldList){
            id obj=[rs objectForColumnName: mapkey];
            if([obj isEqual:[NSNull null]]){
                continue;
            }
            [model setValue:obj forKey:mapkey];
        }
        [modles addObject:model];
    }
    [rs close];
    if(modles.count>0){
        return  modles;
    }
    return nil;
}
/*fetch MAX(column) By Primary Key Field*/
- (id)findMaxKey
{
    NSInteger keyCount = [self.keyList count];
    if(keyCount<=0){
        DLog(@"table no primary key field!");
        return @(1);
    }
    NSString *sql = [NSString stringWithFormat:@"SELECT max(%@) FROM  %@  ",self.keyList[0],self.tableName];
    DLog(@"findMaxKey sql=%@",sql);
    __block id maxValue = @(1);
    [self.queue inDatabase:^(FMDatabase *db){
        FMResultSet *rs = [db executeQuery:sql ];
        if ([rs next]) {
            maxValue = [rs objectForColumnIndex:0];
            [rs close];
        }
        [rs close];
    }];
    return maxValue;
}
@end
